# Fault localization commands

## Slice-based command

Script: fault_localization_slice.py
Required parameters:
 - -i : path to the .slice.csv.xz file
 - -c : path to the change-set.json
 - -b: bugID
 - -f: path to the fails.txt file

Optional parameters:
 - -m: method of the rank calculation (min, average or max, default: average)
 - -a: fault localization algorithms (list, default: Barinel, DStar, Jaccard, Ochiai, SorensenDice and Tarantula)

Example:
```console
user@home:~$ python fault_localization_slice.py -i ../Data/Time-26b-slice.csv.xz -c ../Data/changeset.json -b 26 -f ../SlicingFL/joda-time-26b-fails.txt
Read the (slice) matrix
Read the buggy statements
Read the test- and the statement-names
Convert (matrix) to graph
Calculate the spectra
Calculate the score
Calculate the ranks
Barinel;DStar;Jaccard;Ochiai;SorensenDice;Tarantula
20.5;10.5;19.5;15.5;19.5;20.5
```

## Coverage-based command

Script: fault_localization_coverage.py
Required parameters:
 - -i : path to the .coverage.csv.xz file
 - -c : path to the change-set.json
 - -b: bugID

Optional parameters:
 - -m: method of the rank calculation (min, average or max, default: average)
 - -a: fault localization algorithms (list, default: Barinel, DStar, Jaccard, Ochiai, SorensenDice and Tarantula)

Example:
```console
user@home:~$ python fault_localization_coverage.py -i ../Data/Time-26b-coverage.csv.xz -c ../Data/changeset.json -b 26
Read the (coverage) matrix
Read the buggy statements
Read the test- and the statement-names
Convert (matrix) to graph
Calculate the spectra
Calculate the score
Calculate the ranks
Barinel;DStar;Jaccard;Ochiai;SorensenDice;Tarantula
156.5;60.5;132.5;71.5;132.5;156.5
```

# "Info commands"

## Slice-based command

Script: info_slice.py
Required parameters:
 - -i : path to the .slice.csv.xz file
 - -c : path to the change-set.json
 - -b: bugID
 - -f: path to the fails.txt file

Optional parameters:
 - -m: method of the rank calculation (min, average or max, default: average)
 - -a: fault localization algorithms (list, default: Barinel, DStar, Jaccard, Ochiai, SorensenDice and Tarantula)


Example:
```console
user@home:~$ python info_slice.py -i ../Data/Time-4b-slice.csv.xz -c ../Data/changeset.json -b 4 -f ../SlicingFL/joda-time-4b-fails.txt
Read the (slice) matrix
Read the buggy statements
Read the test- and the statement-names
Convert (matrix) to graph
Calculate the spectra
Calculate the score
Calculate the ranks
The name of the test or statement elements (exit -> 'Exit') :
>> org.joda.time.Partial:464
org.joda.time.Partial:464
    is buggy? -  yes
    ef -  1
    ep -  24
    nf -  0
    np -  3752
    score:
         Barinel  -  0.04
         DStar  -  0.041666666666666664
         Jaccard  -  0.04
         Ochiai  -  0.2
         SorensenDice  -  0.07692307692307693
         Tarantula  -  0.9936842105263157
    rank:
         Barinel  -  1.5
         DStar  -  1.5
         Jaccard  -  1.5
         Ochiai  -  1.5
         SorensenDice  -  1.5
         Tarantula  -  1.5
    covered by tests:
         org.joda.time.TestDateTimeUtils.testIsContiguous_RP
         org.joda.time.TestPartial_Basics.testWith3a
         ...
>> org.joda.time.TestPartial_Basics.testWith3
org.joda.time.TestPartial_Basics.testWith3
    is failed test? -  yes
    covered statements:
         org.joda.time.field.DividedDateTimeField:83  -  not buggy
         org.joda.time.chrono.AssembledChronology:375  -  not buggy
         ...
>> Exit
```

## Coverage-based command

Script: info_coverage.py
Required parameters:
 - -i : path to the .coverage.csv.xz file
 - -c : path to the change-set.json
 - -b: bugID

Optional parameters:
 - -m: method of the rank calculation (min, average or max, default: average)
 - -a: fault localization algorithms (list, default: Barinel, DStar, Jaccard, Ochiai, SorensenDice and Tarantula)

Example:
```console
user@home:~$ python info_coverage.py -i ../Data/Time-4b-coverage.csv.xz -c ../Data/changeset.json -b 4
Read the (coverage) matrix
Read the buggy statements
Read the test- and the statement-names
Convert (matrix) to graph
Calculate the spectra
Calculate the score
Calculate the ranks
The name of the test or statement elements (exit -> 'Exit') :
>> org.joda.time.Partial:464
org.joda.time.Partial:464
    is buggy? -  yes
    ef -  1
    ep -  24
    nf -  0
    np -  3961
    score:
         Barinel  -  0.04
         DStar  -  0.041666666666666664
         Jaccard  -  0.04
         Ochiai  -  0.2
         SorensenDice  -  0.07692307692307693
         Tarantula  -  0.9940134696931904
    rank:
         Barinel  -  23.5
         DStar  -  20.5
         Jaccard  -  23.5
         Ochiai  -  23.5
         SorensenDice  -  23.5
         Tarantula  -  23.5
    covered by tests:
         org.joda.time.TestDateTimeUtils.testIsContiguous_RP
         org.joda.time.TestDateTimeUtils.testIsContiguous_RP_GJChronology
         org.joda.time.TestLocalDateTime_Basics.testCompareTo
         ...
>> org.joda.time.TestPartial_Basics.testWith3
org.joda.time.TestPartial_Basics.testWith3
    is failed test? -  yes
    covered statements:
         org.joda.time.DateTimeZone:134  -  not buggy
         org.joda.time.DateTimeZone:135  -  not buggy
         org.joda.time.DateTimeZone:162  -  not buggy
         org.joda.time.DateTimeZone:175  -  not buggy
         ...
>> Exit
```
