#!/bin/bash

ASSERTFILE=""
JARFILE=""
PREFIX="slice"
KEEPTRYING="no"
EXCLUDE=""

while [ -n "$1" ]
do
    case "$1" in
    -a|--assertfile)
        shift 1
        ASSERTFILE="$1"
    ;;
    -j|--jar)
        shift 1
        JARFILE="$1"
    ;;
    -k|--keep-trying)
        KEEPTRYING="yes"
    ;;
    -p|--prefix)
        shift 1
        PREFIX="$1"
    ;;
    -x|--exclude-list)
        shift 1
        EXCLUDE="$1"
    ;;
    *)
        echo "Unknown parameter: $1"
        exit 1
    ;;
    esac
    shift 1
done

if [ -z "$ASSERTFILE" -o ! -f "$ASSERTFILE" ]
then
    echo "ASSERTFILE is missing"
    exit 1
fi
if [ -z "$JARFILE" -o ! -f "$JARFILE" ]
then
    echo "JARFILE is missing"
    exit 1
fi

function eta {
    STARTTIME=$1
    NOWTIME=$(date +%s)
    NOW=$(date "+%Y-%m-%d %H:%M:%S" -d "@$NOWTIME")
    WORKDONE=$2
    WORKALL=$3
    if [ $WORKDONE == 0 ]
    then
        echo -e "\033[0;30;45m[$NOW] $WORKDONE/$WORKALL\033[0m\033[K"
    else
        ENDTIME=$((STARTTIME + ( ( NOWTIME - STARTTIME ) * WORKALL / WORKDONE ) ))
        echo -ne "\033[0;30;45m[$NOW] $WORKDONE/$WORKALL; ETA: " ; date -d "@$ENDTIME" | tr -d '\012' ; echo -e "\033[0m\033[K"
    fi
}

TOTAL=$(cat $ASSERTFILE | wc -l)
N=0

START=$(date +%s)

for CRITERIA in $(cat $ASSERTFILE)
do
    eta $START $N $TOTAL
    if [ "$CRITERIA" == "#EXIT" ]
    then
        break
    fi
    if [[ "$CRITERIA" == \#* ]]
    then
        ((TOTAL--))
        continue
    fi
    TESTCLASS=$(echo $CRITERIA | cut -d';' -f1)
    TESTMETHOD=$(echo $CRITERIA | cut -d';' -f2)
    ASSERTLINE=$(echo $CRITERIA | cut -d';' -f3)
    SLICEDELTA=$(echo $CRITERIA | cut -d';' -f4)
    SLICELINE=$((ASSERTLINE - SLICEDELTA))
    OUTDIR=${PREFIX}
    SLICEID=${TESTCLASS}-${TESTMETHOD}-${ASSERTLINE}
    SLICEFILE=${PREFIX}-${SLICEID}.txt
    ERRORFILE=${PREFIX}-${SLICEID}.err
    SKIP_FILE=${PREFIX}-${SLICEID}.skp

    if [ -f ${OUTDIR}/${SLICEFILE} -o -f ${OUTDIR}/${ERRORFILE} ]
    then
        echo -e "[${OUTDIR}]: Files for slice ${SLICEID} exist, skipping\033[K\033[2A"
        ((TOTAL--))
        continue
    fi

    if [ -n "${EXCLUDE}" ] && grep -q -e ${SLICEID} ${EXCLUDE}
    then
        echo -e "\033[0;1;33mSKIPPING SLICE COMPUTATION for ${SLICEFILE}\033[0m\033[K"
        date "+%Y-%m-%d %H:%M:%S" >${OUTDIR}/${SKIP_FILE}
        ((TOTAL--))
        continue
    fi
    rm -f ${OUTDIR}/${SKIP_FILE}

    while true
    do
        python3 Slicer4J/scripts/slicer4j.py -j ${JARFILE} -o ${OUTDIR} -tc ${TESTCLASS} -tm ${TESTMETHOD} -b ${TESTCLASS}:${SLICELINE}
        if [ "${KEEPTRYING}" == "no" -o -f ${OUTDIR}/${JARFILE/.jar/_i.jar} ]
        then
            break
        fi
        echo -e "\033[0;1;31mInstrumentation failed for ${JARFILE}, retry\033[0m\033[K"
    done

    if cd ${OUTDIR}
    then
        # grab the slice
        if [ -f slice.log ]
        then
            sort -u slice.log >${SLICEFILE}
            rm slice.log
        else
            echo -e "\033[0;1;31mNO SLICE WAS PRODUCED for ${SLICEFILE}\033[0m\033[K"
            date "+%Y-%m-%d %H:%M:%S" >${ERRORFILE}
        fi

        # cleanup
        #rm -rf jimple_code *_i.jar *.log *.csv instr-classes.txt jar-size.txt

        cd ..
    fi
    ((N++))
done
echo -e "\033[K"
echo -e "Job finished\033[K"
